@@ -1,48 +1,61 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {argMax, argMin, parseArgMinMaxAttributes} from './ops/argminmax';
-import {attention} from './ops/attention';
-import {batchNorm} from './ops/batch-norm';
-import {biasAdd} from './ops/bias-add';
-import {biasSplitGelu} from './ops/bias-split-gelu';
+import { argMax, argMin, parseArgMinMaxAttributes } from './ops/argminmax';
+import { attention } from './ops/attention';
+import { batchNorm } from './ops/batch-norm';
+import { biasAdd } from './ops/bias-add';
+import { biasSplitGelu } from './ops/bias-split-gelu';
 import * as binaryOps from './ops/binary-op';
-import {concat, parseConcatAttributes} from './ops/concat';
-import {conv, parseConvAttributes} from './ops/conv';
-import {convTranspose, parseConvTransposeAttributes} from './ops/conv-transpose';
-import {cumsum, parseCumSumAttributes} from './ops/cumsum';
-import {depthToSpace, parseDepthToSpaceAttributes} from './ops/depth-to-space';
-import {einsum, parseEinsumAttributes} from './ops/einsum';
-import {expand} from './ops/expand';
-import {fastGelu} from './ops/fast-gelu';
-import {gather, parseGatherAttributes} from './ops/gather';
-import {gatherElements, parseGatherElementsAttributes} from './ops/gather-elements';
-import {gemm, parseGemmAttributes} from './ops/gemm';
-import {groupQueryAttention, parseGroupQueryAttentionAttributes} from './ops/group-query-attention';
-import {instanceNorm} from './ops/instance-norm';
-import {layerNorm} from './ops/layer-norm';
-import {matMul} from './ops/matmul';
-import {matMulNBits, parseMatMulNBitsAttributes} from './ops/matmulnbits';
-import {multiHeadAttention, parseMultiHeadAttentionAttributes} from './ops/multihead-attention';
-import {pad} from './ops/pad';
+import { concat, parseConcatAttributes } from './ops/concat';
+import { conv, parseConvAttributes } from './ops/conv';
+import { convTranspose, parseConvTransposeAttributes } from './ops/conv-transpose';
+import { cumsum, parseCumSumAttributes } from './ops/cumsum';
+import { depthToSpace, parseDepthToSpaceAttributes } from './ops/depth-to-space';
+import { einsum, parseEinsumAttributes } from './ops/einsum';
+import { expand } from './ops/expand';
+import { fastGelu } from './ops/fast-gelu';
+import { gather, parseGatherAttributes } from './ops/gather';
+import { gatherBlockQuantized, parseGatherBlockQuantizedAttributes } from './ops/gather-block-quantized';
+import { gatherElements, parseGatherElementsAttributes } from './ops/gather-elements';
+import { gemm, parseGemmAttributes } from './ops/gemm';
+import { groupQueryAttention, parseGroupQueryAttentionAttributes } from './ops/group-query-attention';
+import { instanceNorm } from './ops/instance-norm';
+import { layerNorm } from './ops/layer-norm';
+import { matMul } from './ops/matmul';
+import { matMulNBits, parseMatMulNBitsAttributes } from './ops/matmulnbits';
+import { multiHeadAttention, parseMultiHeadAttentionAttributes } from './ops/multihead-attention';
+import { pad } from './ops/pad';
 import * as pool from './ops/pool';
-import {range} from './ops/range';
-import {reduceL1, reduceL2, reduceLogSum, reduceLogSumExp, reduceMax, reduceMean, reduceMin, reduceProd, reduceSum, reduceSumSquare} from './ops/reduce';
-import {parseResizeAttributes, resize} from './ops/resize';
-import {rotaryEmbedding} from './ops/rotary-embedding';
-import {skipLayerNorm} from './ops/skip-layer-norm';
-import {parseSliceAttributes, slice} from './ops/slice';
-import {parseSoftmaxAttributes, softmax} from './ops/softmax';
-import {parseSplitAttributes, split} from './ops/split';
-import {tile} from './ops/tile';
-import {parseTransposeAttributes, transpose} from './ops/transpose';
+import { dequantizeLinear, parseDequantizeLinearAttributes } from './ops/quantize-linear';
+import { range } from './ops/range';
+import {
+  reduceL1,
+  reduceL2,
+  reduceLogSum,
+  reduceLogSumExp,
+  reduceMax,
+  reduceMean,
+  reduceMin,
+  reduceProd,
+  reduceSum,
+  reduceSumSquare,
+} from './ops/reduce';
+import { parseResizeAttributes, resize } from './ops/resize';
+import { rotaryEmbedding } from './ops/rotary-embedding';
+import { skipLayerNorm } from './ops/skip-layer-norm';
+import { parseSliceAttributes, slice } from './ops/slice';
+import { parseSoftmaxAttributes, softmax } from './ops/softmax';
+import { parseSplitAttributes, split } from './ops/split';
+import { tile } from './ops/tile';
+import { parseTransposeAttributes, transpose } from './ops/transpose';
 import * as unaryOps from './ops/unary-op';
-import {where} from './ops/where';
-import {ComputeContext} from './types';
+import { where } from './ops/where';
+import { ComputeContext } from './types';
 
 export type RunFunction = (context: ComputeContext, attribute?: unknown) => void;
 export type ParseAttributeFunction = (attributeRaw: unknown) => unknown;
-export type OperatorImplementation = [RunFunction]|[RunFunction, ParseAttributeFunction];
+export type OperatorImplementation = [RunFunction] | [RunFunction, ParseAttributeFunction];
 
 export const WEBGPU_OP_RESOLVE_RULES: Map<string, OperatorImplementation> = new Map([
   ['Abs', [unaryOps.abs]],
@@ -71,6 +84,7 @@ export const WEBGPU_OP_RESOLVE_RULES: Map<string, OperatorImplementation> = new
   ['Cosh', [unaryOps.cosh]],
   ['CumSum', [cumsum, parseCumSumAttributes]],
   ['DepthToSpace', [depthToSpace, parseDepthToSpaceAttributes]],
+  ['DequantizeLinear', [dequantizeLinear, parseDequantizeLinearAttributes]],
   ['Div', [binaryOps.div]],
   ['Einsum', [einsum, parseEinsumAttributes]],
   ['Elu', [unaryOps.elu, unaryOps.parseAlphaAttributes]],
@@ -83,6 +97,7 @@ export const WEBGPU_OP_RESOLVE_RULES: Map<string, OperatorImplementation> = new
   ['FusedConv', [conv, parseConvAttributes]],
   ['Gather', [gather, parseGatherAttributes]],
   ['GatherElements', [gatherElements, parseGatherElementsAttributes]],
+  ['GatherBlockQuantized', [gatherBlockQuantized, parseGatherBlockQuantizedAttributes]],
   ['Gelu', [unaryOps.gelu]],
   ['Gemm', [gemm, parseGemmAttributes]],
   ['GlobalAveragePool', [pool.globalAveragePool, pool.parseGlobalAveragePoolAttributes]],
