@@ -1,35 +1,45 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {DataType} from '../../../wasm-common';
-import {TensorView} from '../../tensor-view';
-import {ComputeContext, GpuDataType, ProgramInputTensorInfoDependency, ProgramUniform} from '../types';
+import { DataType } from '../../../wasm-common';
+import { TensorView } from '../../tensor-view';
+import { ShapeUtil } from '../../util';
+import { ComputeContext, GpuDataType, ProgramInputTensorInfoDependency, ProgramUniform } from '../types';
 
-import {getMaxComponents, inputVariable, outputVariable, ShaderHelper, tensorTypeToWsglStorageType, tensorTypeToWsglValueType, UniformDataElementType, UniformsArrayType} from './common';
+import {
+  getMaxComponents,
+  inputVariable,
+  outputVariable,
+  ShaderHelper,
+  tensorTypeToWsglStorageType,
+  tensorTypeToWsglValueType,
+  UniformDataElementType,
+  UniformsArrayType,
+} from './common';
 
 export const enum AttentionQkvFormat {
-  unknown,          // enum value not set, or depends on qkv projection implementation details
-  qkvBNSH,          // for non-packed qkv, permuted
-  qkvBSNH,          // for non-packed qkv, not permuted, used by memory efficient attention or MultiHeadAttention
-  qkvBSN3H,         // for TRT fused attention, qkv are packed
-  qkvBNSHqkvBS3NH,  // for TRT fused causal attention, data has two formats (qkv is 3BNSH, gemm_buffer is BS3NH)
-  qKvBSNHxBSN2H,    // for TRT fused cross attention, kv are packed
-  qkvTNH,           // for memory efficient attention, qkv are not packed, and paddings are removed.
-  qkvTN3H,          // for TRT fused attention, qkv are packed and paddings are removed
+  unknown, // enum value not set, or depends on qkv projection implementation details
+  qkvBNSH, // for non-packed qkv, permuted
+  qkvBSNH, // for non-packed qkv, not permuted, used by memory efficient attention or MultiHeadAttention
+  qkvBSN3H, // for TRT fused attention, qkv are packed
+  qkvBNSHqkvBS3NH, // for TRT fused causal attention, data has two formats (qkv is 3BNSH, gemm_buffer is BS3NH)
+  qKvBSNHxBSN2H, // for TRT fused cross attention, kv are packed
+  qkvTNH, // for memory efficient attention, qkv are not packed, and paddings are removed.
+  qkvTN3H, // for TRT fused attention, qkv are packed and paddings are removed
 }
 
 export const enum AttentionMaskType {
-  none,                  // No mask
-  mask1dKeySeqLen,       // [batch_size], key sequence length
-  mask1dEndStart,        // [2 * batch_size] with end positions and start positions
-  mask1DKeySeqLenStart,  // [3 * batch_size + 2] with [key_len[0], ..., key_len[batch_size - 1], query_start[0],
-                         // ..., query_start[batch_size - 1], query_end[batch_size - 1], key_start[0], ...,
-                         // key_start[batch_size - 1], key_end[batch_size - 1]]
-  mask2dDummy,           // dummy mask with shape [1, 1] or [batch_size, 1]. It has same effect as no mask.
-  mask2dKeyPadding,      // [batch_size, total_sequence_length]
-  mask3dAttention,       // [batch_size, sequence_length, total_sequence_length]
-  mask4dMegatron,        // Megatron causal mask with shape [batch_size, 1, max_sequence_length, max_sequence_length]
-  maskUnknown
+  none, // No mask
+  mask1dKeySeqLen, // [batch_size], key sequence length
+  mask1dEndStart, // [2 * batch_size] with end positions and start positions
+  mask1DKeySeqLenStart, // [3 * batch_size + 2] with [key_len[0], ..., key_len[batch_size - 1], query_start[0],
+  // ..., query_start[batch_size - 1], query_end[batch_size - 1], key_start[0], ...,
+  // key_start[batch_size - 1], key_end[batch_size - 1]]
+  mask2dDummy, // dummy mask with shape [1, 1] or [batch_size, 1]. It has same effect as no mask.
+  mask2dKeyPadding, // [batch_size, total_sequence_length]
+  mask3dAttention, // [batch_size, sequence_length, total_sequence_length]
+  mask4dMegatron, // Megatron causal mask with shape [batch_size, 1, max_sequence_length, max_sequence_length]
+  maskUnknown,
 }
 
 export interface AttentionParameters {
@@ -92,7 +102,7 @@ const validateAttentionInputs = (inputs: readonly TensorView[], attributes: Atte
   //   bias         (Q/K/V)    : (D + D + D_v)
   //   mask_index              : see below
   //   past         (K/V)      : (2, B, N, P, H) or NULL
-  //   relative_position_bias            : (B, N, S, T) or NULL
+  //   attention_bias          : (B, N, S, T) or NULL
 
   // For mask_index, the following shapes are supported:
   //     NULL, (B, 1), (1, 1)
@@ -109,10 +119,10 @@ const validateAttentionInputs = (inputs: readonly TensorView[], attributes: Atte
   const bias = inputs[2];
   const maskIndex = inputs[3];
   const past = inputs[4];
-  const relativePositionBias = inputs[5];
+  const attentionBias = inputs[5];
 
-  if (past && relativePositionBias) {
-    throw new Error('Attention cannot have both past and relative_position_bias');
+  if (past && attentionBias) {
+    throw new Error('Attention cannot have both past and attention_bias');
   }
 
   if (input.dims.length !== 3) {
@@ -208,6 +218,22 @@ const validateAttentionInputs = (inputs: readonly TensorView[], attributes: Atte
     throw new Error('past is not supported');
   }
 
+  if (attentionBias) {
+    if (attentionBias.dims.length !== 4) {
+      throw new Error('Input "attention_bias" must have 4 dimensions');
+    }
+
+    // TODO: support broadcasting the first and second dimensions of attention_bias
+    if (
+      attentionBias.dims[0] !== batchSize ||
+      attentionBias.dims[1] !== attributes.numHeads ||
+      attentionBias.dims[2] !== sequenceLength ||
+      attentionBias.dims[3] !== totalSequenceLength
+    ) {
+      throw new Error('Expect "attention_bias" shape (batch_size, num_heads, sequence_length, total_sequence_length)');
+    }
+  }
+
   return {
     batchSize,
     sequenceLength,
@@ -232,40 +258,38 @@ const validateAttentionInputs = (inputs: readonly TensorView[], attributes: Atte
   };
 };
 
-const createInPlaceSoftmaxProgramInfo = (_context: ComputeContext, input: TensorView, n: number, d: number) => {
+const createInPlaceSoftmaxProgramInfo = (input: TensorView, n: number, d: number) => {
   const components = getMaxComponents(d);
   let WG = 64;
   const dComp = d / components;
   if (dComp < WG) {
-    WG = 1;
-  } else if (dComp / 8 < 64) {
-    WG = Math.ceil(dComp / 8);
+    WG = 32;
   }
   const elementsPerThread = Math.ceil(d / components / WG);
   const programUniforms: ProgramUniform[] = [
-    {type: input.dataType, data: 1 / d}, {type: DataType.uint32, data: dComp},
-    {type: DataType.uint32, data: elementsPerThread}
+    { type: DataType.float, data: 1 / d },
+    { type: DataType.uint32, data: dComp },
+    { type: DataType.uint32, data: elementsPerThread },
   ];
   const dataType = tensorTypeToWsglStorageType(input.dataType, components);
   const f32Type = tensorTypeToWsglValueType(DataType.float, components);
-
+  const inputDependencies: ProgramInputTensorInfoDependency[] = ['type'];
   const getShaderSource = (shaderHelper: ShaderHelper) => {
     const inputHelper = outputVariable('x', input.dataType, input.dims, components);
     const elemValueType = tensorTypeToWsglValueType(input.dataType);
     const uniforms: UniformsArrayType = [
-      {name: 'd_inv', type: elemValueType as UniformDataElementType}, {name: 'd_comp', type: 'u32'},
-      {name: 'elements_per_thread', type: 'u32'}
+      { name: 'd_inv', type: 'f32' },
+      { name: 'd_comp', type: 'u32' },
+      { name: 'elements_per_thread', type: 'u32' },
     ];
 
     return `
   var<workgroup> thread_max: array<f32, ${WG}>;
   var<workgroup> thread_sum: array<f32, ${WG}>;
   ${shaderHelper.registerUniforms(uniforms).declareVariables(inputHelper)}
-  ${shaderHelper.mainStart([
-      WG, 1, 1
-    ])}
+  ${shaderHelper.mainStart([WG, 1, 1])}
     let local_offset = local_idx * uniforms.elements_per_thread;
-    let offset = workgroup_id.x * uniforms.d_comp + local_offset;
+    let offset = (global_idx / ${WG}) * uniforms.d_comp + local_offset;
 
     var thread_max_vector = ${f32Type}(-3.402823e+38f);
     for (var i: u32 = 0; i < uniforms.elements_per_thread && i + local_offset < uniforms.d_comp; i++) {
@@ -315,7 +339,7 @@ const createInPlaceSoftmaxProgramInfo = (_context: ComputeContext, input: Tensor
 
     if (sum == 0) {
       for (var i: u32 = 0; i < uniforms.elements_per_thread && i + local_offset < uniforms.d_comp; i++) {
-        x[offset + i] = ${inputHelper.type.value}(uniforms.d_inv);
+        x[offset + i] = ${inputHelper.type.value}(${elemValueType}(uniforms.d_inv));
       }
     } else {
       for (var i: u32 = 0; i < uniforms.elements_per_thread && i + local_offset < uniforms.d_comp; i++) {
@@ -328,100 +352,111 @@ const createInPlaceSoftmaxProgramInfo = (_context: ComputeContext, input: Tensor
 
   return {
     name: 'AttentionProbsSoftmax',
-    shaderCache: {hint: `${WG};${dataType};${components}`},
+    shaderCache: { hint: `${WG};${dataType};${components}`, inputDependencies },
     getShaderSource,
-    getRunData: () => ({outputs: [], dispatchGroup: {x: n}, programUniforms}),
+    getRunData: () => ({ outputs: [], dispatchGroup: { x: n }, programUniforms }),
   };
 };
 
-const createAttentionProbsProgramInfo =
-    (context: ComputeContext, q: TensorView, key: TensorView, pastKey: TensorView|undefined,
-     relativePositionBias: TensorView|undefined, parameters: AttentionParameters, attributes: AttentionAttrs,
-     pastSequenceLength: number) => {
-      const totalSequenceLength = pastSequenceLength + parameters.kvSequenceLength;
-      const probsShape = [parameters.batchSize, parameters.numHeads, parameters.sequenceLength, totalSequenceLength];
-      const presentKey = parameters.kvNumHeads === undefined && context.outputCount > 1;
-      const presentKeyShape = presentKey ?
-          [parameters.batchSize, parameters.numHeads, totalSequenceLength, parameters.headSize] :
-          undefined;
+const createAttentionProbsProgramInfo = (
+  outputCount: number,
+  q: TensorView,
+  key: TensorView,
+  pastKey: TensorView | undefined,
+  attentionBias: TensorView | undefined,
+  parameters: AttentionParameters,
+  attributes: AttentionAttrs,
+  pastSequenceLength: number,
+) => {
+  const totalSequenceLength = pastSequenceLength + parameters.kvSequenceLength;
+  const probsShape = [parameters.batchSize, parameters.numHeads, parameters.sequenceLength, totalSequenceLength];
+  const presentKey = parameters.kvNumHeads === undefined && outputCount > 1 && pastKey;
+  const presentKeyShape = presentKey
+    ? [parameters.batchSize, parameters.numHeads, totalSequenceLength, parameters.headSize]
+    : undefined;
 
-      // TODO: handle mask
+  // TODO: handle mask
 
-      const alpha = attributes.scale === 0 ? 1.0 / Math.sqrt(parameters.headSize) : attributes.scale;
-      const components = getMaxComponents(parameters.headSize);
-      const vectorizedHeadSize = parameters.headSize / components;
-      const TILE_SIZE = 12;
-      const dispatch = {
-        x: Math.ceil(totalSequenceLength / TILE_SIZE),
-        y: Math.ceil(parameters.sequenceLength / TILE_SIZE),
-        z: parameters.batchSize * parameters.numHeads
-      };
-      const programUniforms: ProgramUniform[] = [
-        {type: DataType.uint32, data: parameters.sequenceLength}, {type: DataType.uint32, data: vectorizedHeadSize},
-        {type: DataType.uint32, data: totalSequenceLength}, {type: DataType.uint32, data: parameters.numHeads},
-        {type: DataType.float, data: alpha}, {type: DataType.uint32, data: pastSequenceLength},
-        {type: DataType.uint32, data: parameters.kvSequenceLength}
-      ];
+  const alpha = attributes.scale === 0 ? 1.0 / Math.sqrt(parameters.headSize) : attributes.scale;
+  const components = getMaxComponents(parameters.headSize);
+  const vectorizedHeadSize = parameters.headSize / components;
+  const TILE_SIZE = 12;
+  const dispatch = {
+    x: Math.ceil(totalSequenceLength / TILE_SIZE),
+    y: Math.ceil(parameters.sequenceLength / TILE_SIZE),
+    z: parameters.batchSize * parameters.numHeads,
+  };
+  const programUniforms: ProgramUniform[] = [
+    { type: DataType.uint32, data: parameters.sequenceLength },
+    { type: DataType.uint32, data: vectorizedHeadSize },
+    { type: DataType.uint32, data: totalSequenceLength },
+    { type: DataType.uint32, data: parameters.numHeads },
+    { type: DataType.float, data: alpha },
+    { type: DataType.uint32, data: pastSequenceLength },
+    { type: DataType.uint32, data: parameters.kvSequenceLength },
+  ];
+  // Feed pastKey to the shader-code only if it is non-zero and presentKey is being produced
+  const feedPastKey = presentKey && pastKey && ShapeUtil.size(pastKey.dims) > 0;
+  const inputDependencies: ProgramInputTensorInfoDependency[] = ['type', 'type'];
+  if (feedPastKey) {
+    inputDependencies.push('type');
+  }
+  if (attentionBias) {
+    inputDependencies.push('type');
+  }
+  const outputs = [{ dims: probsShape, dataType: q.dataType, gpuDataType: GpuDataType.default }];
+  if (presentKey) {
+    outputs.push({ dims: presentKeyShape!, dataType: q.dataType, gpuDataType: GpuDataType.default });
+  }
+  const getShaderSource = (shaderHelper: ShaderHelper) => {
+    const qInput = inputVariable('q', q.dataType, q.dims, components);
+    const kInput = inputVariable('key', key.dataType, key.dims, components);
+    const inputVars = [qInput, kInput];
+    if (feedPastKey) {
+      const pastKeyInput = inputVariable('past_key', pastKey.dataType, pastKey.dims, components);
+      inputVars.push(pastKeyInput);
+    }
+    if (attentionBias) {
+      inputVars.push(inputVariable('attention_bias', attentionBias.dataType, attentionBias.dims));
+    }
+    const output = outputVariable('output', q.dataType, probsShape);
+    const outputVars = [output];
+    if (presentKey) {
+      outputVars.push(outputVariable('present_key', q.dataType, presentKeyShape!, components));
+    }
+    const f32Type = tensorTypeToWsglValueType(DataType.float, components);
 
-      const inputDependencies: ProgramInputTensorInfoDependency[] = ['type', 'type'];
-      if (pastKey) {
-        inputDependencies.push('type');
-      }
-      if (relativePositionBias) {
-        inputDependencies.push('type');
-      }
-      const outputs = [{dims: probsShape, dataType: q.dataType, gpuDataType: GpuDataType.default}];
-      if (presentKey) {
-        outputs.push({dims: presentKeyShape!, dataType: q.dataType, gpuDataType: GpuDataType.default});
-      }
-      const getShaderSource = (shaderHelper: ShaderHelper) => {
-        const qInput = inputVariable('q', q.dataType, q.dims, components);
-        const kInput = inputVariable('key', key.dataType, key.dims, components);
-        const inputVars = [qInput, kInput];
-        if (pastKey) {
-          const pastKeyInput = inputVariable('past_key', pastKey.dataType, pastKey.dims, components);
-          inputVars.push(pastKeyInput);
-        }
-        if (relativePositionBias) {
-          inputVars.push(
-              inputVariable('relative_position_bias', relativePositionBias.dataType, relativePositionBias.dims));
-        }
-        const output = outputVariable('output', q.dataType, probsShape);
-        const outputVars = [output];
-        if (presentKey) {
-          outputVars.push(outputVariable('present_key', q.dataType, presentKeyShape!, components));
-        }
-        const f32Type = tensorTypeToWsglValueType(DataType.float, components);
-
-        const uniforms: UniformsArrayType = [
-          {name: 'M', type: 'u32'}, {name: 'K', type: 'u32'}, {name: 'N', type: 'u32'},
-          {name: 'num_heads', type: 'u32'}, {name: 'alpha', type: 'f32' as UniformDataElementType},
-          {name: 'past_sequence_length', type: 'u32'}, {name: 'kv_sequence_length', type: 'u32'}
-        ];
-        return `
+    const uniforms: UniformsArrayType = [
+      { name: 'M', type: 'u32' },
+      { name: 'K', type: 'u32' },
+      { name: 'N', type: 'u32' },
+      { name: 'num_heads', type: 'u32' },
+      { name: 'alpha', type: 'f32' as UniformDataElementType },
+      { name: 'past_sequence_length', type: 'u32' },
+      { name: 'kv_sequence_length', type: 'u32' },
+    ];
+    return `
   const TILE_SIZE = ${TILE_SIZE}u;
 
   var<workgroup> tileQ: array<${qInput.type.storage}, ${TILE_SIZE * TILE_SIZE}>;
   var<workgroup> tileK: array<${qInput.type.storage}, ${TILE_SIZE * TILE_SIZE}>;
   ${shaderHelper.registerUniforms(uniforms).declareVariables(...inputVars, ...outputVars)}
-  ${shaderHelper.mainStart([
-          TILE_SIZE, TILE_SIZE, 1
-        ])}
+  ${shaderHelper.mainStart([TILE_SIZE, TILE_SIZE, 1])}
     // x holds the N and y holds the M
     let headIdx = workgroup_id.z;
     let m = workgroup_id.y * TILE_SIZE;
     let n = workgroup_id.x * TILE_SIZE;
     let qOffset = uniforms.M * uniforms.K * headIdx + m * uniforms.K;
     ${(() => {
-          if (pastKey && presentKey) {
-            return `
+      if (feedPastKey && presentKey) {
+        return `
     let kOffset = uniforms.kv_sequence_length * uniforms.K * headIdx;
     let pastKeyOffset = uniforms.past_sequence_length * uniforms.K * headIdx;`;
-          } else {
-            return `
+      } else {
+        return `
     let kOffset = uniforms.N * uniforms.K * headIdx + n * uniforms.K;`;
-          }
-        })()}
+      }
+    })()}
     ${presentKey ? 'let presentKeyOffset = headIdx * uniforms.N * uniforms.K;' : ''}
     var value = ${f32Type}(0);
     for (var w: u32 = 0u; w < uniforms.K; w += TILE_SIZE) {
@@ -431,22 +466,21 @@ const createAttentionProbsProgramInfo =
       if (n + local_id.y < uniforms.N && w + local_id.x < uniforms.K) {
         var idx = TILE_SIZE * local_id.y + local_id.x;
       ${(() => {
-          if (pastKey && presentKey) {
-            return `
+        if (feedPastKey && presentKey) {
+          return `
               if (n + local_id.y < uniforms.past_sequence_length) {
                 tileK[idx] = past_key[pastKeyOffset + (n + local_id.y) * uniforms.K + w + local_id.x];
               } else {
                 tileK[idx] =
                          key[kOffset + (n + local_id.y - uniforms.past_sequence_length) * uniforms.K + w + local_id.x];
               }`;
-          } else {
-            return 'tileK[idx] = key[kOffset + local_id.y * uniforms.K + w + local_id.x];';
-          }
-        })()}
+        } else {
+          return 'tileK[idx] = key[kOffset + local_id.y * uniforms.K + w + local_id.x];';
+        }
+      })()}
       ${
-            presentKey ?
-                'present_key[presentKeyOffset + (n + local_id.y) * uniforms.K + w + local_id.x] = tileK[idx];' :
-                ''}
+        presentKey ? 'present_key[presentKeyOffset + (n + local_id.y) * uniforms.K + w + local_id.x] = tileK[idx];' : ''
+      }
       }
       workgroupBarrier();
 
@@ -461,105 +495,120 @@ const createAttentionProbsProgramInfo =
     if (global_id.y < uniforms.M && global_id.x < uniforms.N) {
       let outputIdx = headOffset + global_id.y * uniforms.N + global_id.x;
       var sum: f32 = ${(() => {
-          switch (components) {
-            case 1:
-              return 'value';
-            case 2:
-              return 'value.x + value.y';
-            case 4:
-              return 'value.x + value.y + value.z + value.w';
-            default:
-              throw new Error(`Unsupported components: ${components}`);
-          }
-        })()};
+        switch (components) {
+          case 1:
+            return 'value';
+          case 2:
+            return 'value.x + value.y';
+          case 4:
+            return 'value.x + value.y + value.z + value.w';
+          default:
+            throw new Error(`Unsupported components: ${components}`);
+        }
+      })()};
         output[outputIdx] = ${output.type.value} (sum * uniforms.alpha) + ${
-            relativePositionBias ? 'relative_position_bias[outputIdx]' : '0.0'};
+          attentionBias ? 'attention_bias[outputIdx]' : '0.0'
+        };
     }
   }`;
-      };
-      return {
-        name: 'AttentionProbs',
-        shaderCache: {
-          hint: `${components};${relativePositionBias !== undefined};${pastKey !== undefined};${context.outputCount}`,
-          inputDependencies
-        },
-        getRunData: () => ({outputs, dispatchGroup: dispatch, programUniforms}),
-        getShaderSource,
-      };
-    };
+  };
+  return {
+    name: 'AttentionProbs',
+    shaderCache: {
+      hint: `${components};${attentionBias !== undefined};${pastKey !== undefined};${outputCount}`,
+      inputDependencies,
+    },
+    getRunData: () => ({ outputs, dispatchGroup: dispatch, programUniforms }),
+    getShaderSource,
+  };
+};
 
+const createVxAttentionScoreProgramInfo = (
+  outputCount: number,
+  probs: TensorView,
+  v: TensorView,
+  pastValue: TensorView | undefined,
+  params: AttentionParameters,
+  pastSequenceLength: number,
+) => {
+  const totalSequenceLength = pastSequenceLength + params.kvSequenceLength;
+  const nReps = params.nReps ? params.nReps : 1;
+  const repeatedVHiddenSize = params.vHiddenSize * nReps;
+  const presentValue = params.kvNumHeads == null && outputCount > 1 && pastValue;
+  const presentValueShape = presentValue
+    ? [params.batchSize, params.numHeads, totalSequenceLength, params.headSize]
+    : undefined;
+  const outputShape = [params.batchSize, params.sequenceLength, repeatedVHiddenSize];
+  const TILE_SIZE = 12;
+  const dispatch = {
+    x: Math.ceil(params.vHeadSize / TILE_SIZE),
+    y: Math.ceil(params.sequenceLength / TILE_SIZE),
+    z: params.batchSize * params.numHeads,
+  };
 
-const createVxAttentionScoreProgramInfo =
-    (context: ComputeContext, probs: TensorView, v: TensorView, pastValue: TensorView|undefined,
-     params: AttentionParameters, pastSequenceLength: number) => {
-      const totalSequenceLength = pastSequenceLength + params.kvSequenceLength;
-      const nReps = params.nReps ? params.nReps : 1;
-      const repeatedVHiddenSize = params.vHiddenSize * nReps;
-      const presentValue = params.kvNumHeads == null && context.outputCount > 1;
-      const presentValueShape =
-          presentValue ? [params.batchSize, params.numHeads, totalSequenceLength, params.headSize] : undefined;
-      const outputShape = [params.batchSize, params.sequenceLength, repeatedVHiddenSize];
-      const TILE_SIZE = 12;
-      const dispatch = {
-        x: Math.ceil(params.vHeadSize / TILE_SIZE),
-        y: Math.ceil(params.sequenceLength / TILE_SIZE),
-        z: params.batchSize * params.numHeads
-      };
-
-      const programUniforms: ProgramUniform[] = [
-        {type: DataType.uint32, data: params.sequenceLength}, {type: DataType.uint32, data: totalSequenceLength},
-        {type: DataType.uint32, data: params.vHeadSize}, {type: DataType.uint32, data: params.numHeads},
-        {type: DataType.uint32, data: repeatedVHiddenSize}, {type: DataType.uint32, data: pastSequenceLength},
-        {type: DataType.uint32, data: params.kvSequenceLength}
-      ];
-      const inputDependencies: ProgramInputTensorInfoDependency[] =
-          pastValue ? ['type', 'type', 'type'] : ['type', 'type'];
-      const outputs = [{dims: outputShape, dataType: probs.dataType, gpuDataType: GpuDataType.default}];
-      if (presentValue) {
-        outputs.push({dims: presentValueShape!, dataType: probs.dataType, gpuDataType: GpuDataType.default});
-      }
-      const getShaderSource = (shaderHelper: ShaderHelper) => {
-        const probsHelper = inputVariable('probs', probs.dataType, probs.dims);
-        const vHelper = inputVariable('v', v.dataType, v.dims);
-        const inputVars = [probsHelper, vHelper];
-        if (pastValue) {
-          inputVars.push(inputVariable('past_value', pastValue.dataType, pastValue.dims));
-        }
-        const output = outputVariable('output', probs.dataType, outputShape);
-        const outputVars = [output];
-        if (presentValue) {
-          outputVars.push(outputVariable('present_value', probs.dataType, presentValueShape!));
-        }
-        const uniforms: UniformsArrayType = [
-          {name: 'M', type: 'u32'}, {name: 'K', type: 'u32'}, {name: 'N', type: 'u32'},
-          {name: 'num_heads', type: 'u32'}, {name: 'v_hidden_size', type: 'u32'},
-          {name: 'past_sequence_length', type: 'u32'}, {name: 'kv_sequence_length', type: 'u32'}
-        ];
-        return `
+  const programUniforms: ProgramUniform[] = [
+    { type: DataType.uint32, data: params.sequenceLength },
+    { type: DataType.uint32, data: totalSequenceLength },
+    { type: DataType.uint32, data: params.vHeadSize },
+    { type: DataType.uint32, data: params.numHeads },
+    { type: DataType.uint32, data: repeatedVHiddenSize },
+    { type: DataType.uint32, data: pastSequenceLength },
+    { type: DataType.uint32, data: params.kvSequenceLength },
+  ];
+  // Feed pastValue to the shader-code only if it is non-empty and presentValue is being produced
+  const feedPastValue = presentValue && pastValue && ShapeUtil.size(pastValue.dims) > 0;
+  const inputDependencies: ProgramInputTensorInfoDependency[] = ['type', 'type'];
+  if (feedPastValue) {
+    inputDependencies.push('type');
+  }
+  const outputs = [{ dims: outputShape, dataType: probs.dataType, gpuDataType: GpuDataType.default }];
+  if (presentValue) {
+    outputs.push({ dims: presentValueShape!, dataType: probs.dataType, gpuDataType: GpuDataType.default });
+  }
+  const getShaderSource = (shaderHelper: ShaderHelper) => {
+    const probsHelper = inputVariable('probs', probs.dataType, probs.dims);
+    const vHelper = inputVariable('v', v.dataType, v.dims);
+    const inputVars = [probsHelper, vHelper];
+    if (feedPastValue) {
+      inputVars.push(inputVariable('past_value', pastValue.dataType, pastValue.dims));
+    }
+    const output = outputVariable('output', probs.dataType, outputShape);
+    const outputVars = [output];
+    if (presentValue) {
+      outputVars.push(outputVariable('present_value', probs.dataType, presentValueShape!));
+    }
+    const uniforms: UniformsArrayType = [
+      { name: 'M', type: 'u32' },
+      { name: 'K', type: 'u32' },
+      { name: 'N', type: 'u32' },
+      { name: 'num_heads', type: 'u32' },
+      { name: 'v_hidden_size', type: 'u32' },
+      { name: 'past_sequence_length', type: 'u32' },
+      { name: 'kv_sequence_length', type: 'u32' },
+    ];
+    return `
   const TILE_SIZE = ${TILE_SIZE}u;
   var<workgroup> tileQ: array<${probsHelper.type.value}, ${TILE_SIZE * TILE_SIZE}>;
   var<workgroup> tileK: array<${probsHelper.type.value}, ${TILE_SIZE * TILE_SIZE}>;
   ${shaderHelper.registerUniforms(uniforms).declareVariables(...inputVars, ...outputVars)}
-  ${shaderHelper.mainStart([
-          TILE_SIZE, TILE_SIZE, 1
-        ])}
+  ${shaderHelper.mainStart([TILE_SIZE, TILE_SIZE, 1])}
    let headIdx = workgroup_id.z;
    let m = global_id.y;
    let n = global_id.x;
 
    let offsetA = headIdx * (uniforms.M * uniforms.K) + m * uniforms.K;
    ${(() => {
-          if (pastValue && presentValue) {
-            return `
+     if (feedPastValue && presentValue) {
+       return `
     let pastValueOffset = headIdx * uniforms.N * uniforms.past_sequence_length + n;
     let vOffset = headIdx * uniforms.N * uniforms.kv_sequence_length + n;
       `;
-          } else {
-            return `
+     } else {
+       return `
    let offsetB = headIdx * uniforms.N * uniforms.K + n;
             `;
-          }
-        })()}
+     }
+   })()}
     ${presentValue ? 'let presentValueOffset = headIdx * uniforms.N * uniforms.K + n;' : ''}
    var value = ${probsHelper.type.storage}(0);
    for (var w: u32 = 0u; w < uniforms.K; w += TILE_SIZE) {
@@ -569,7 +618,7 @@ const createVxAttentionScoreProgramInfo =
       if (n < uniforms.N && w + local_id.y < uniforms.K) {
         var idx = TILE_SIZE * local_id.y + local_id.x;
         ${(() => {
-          if (pastValue && presentValue) {
+          if (feedPastValue && presentValue) {
             return `
         if (w + local_id.y < uniforms.past_sequence_length) {
           tileK[idx] = past_value[pastValueOffset + (w + local_id.y) * uniforms.N];
@@ -601,60 +650,82 @@ const createVxAttentionScoreProgramInfo =
      output[outputIdx] = value;
    }
   }`;
-      };
+  };
 
-      return {
-        name: 'AttentionScore',
-        shaderCache: {hint: `${pastValue !== undefined};${context.outputCount}`, inputDependencies},
-        getRunData: () => ({outputs, dispatchGroup: dispatch, programUniforms}),
-        getShaderSource,
-      };
-    };
+  return {
+    name: 'AttentionScore',
+    shaderCache: { hint: `${pastValue !== undefined};${outputCount}`, inputDependencies },
+    getRunData: () => ({ outputs, dispatchGroup: dispatch, programUniforms }),
+    getShaderSource,
+  };
+};
 
-export const applyAttention =
-    (context: ComputeContext, q: TensorView, k: TensorView, v: TensorView, _maskIndex: TensorView|undefined,
-     _past: TensorView|undefined, pastKey: TensorView|undefined, pastValue: TensorView|undefined,
-     relativePositionBias: TensorView|undefined, parameters: AttentionParameters, attributes: AttentionAttrs) => {
-      const outputCount = context.outputCount;
-      const pastSequenceLength =
-          parameters.kvNumHeads !== undefined || outputCount > 1 ? parameters.pastSequenceLength : 0;
-      const totalSequenceLength = pastSequenceLength + parameters.kvSequenceLength;
+export const applyAttention = (
+  context: ComputeContext,
+  q: TensorView,
+  k: TensorView,
+  v: TensorView,
+  _maskIndex: TensorView | undefined,
+  _past: TensorView | undefined,
+  pastKey: TensorView | undefined,
+  pastValue: TensorView | undefined,
+  attentionBiasInput: TensorView | undefined,
+  parameters: AttentionParameters,
+  attributes: AttentionAttrs,
+) => {
+  // Assumption  is that presentKey/presentValue exists only if pastKey/pastValue exists.
+  const outputCount = Math.min(context.outputCount, 1 + (pastKey ? 1 : 0) + (pastValue ? 1 : 0));
+  const pastSequenceLength = parameters.kvNumHeads !== undefined || outputCount > 1 ? parameters.pastSequenceLength : 0;
+  const totalSequenceLength = pastSequenceLength + parameters.kvSequenceLength;
+  const attentionBias =
+    attentionBiasInput && ShapeUtil.size(attentionBiasInput.dims) > 0 ? attentionBiasInput : undefined;
 
-      const inputsK = (parameters.kvNumHeads === undefined && outputCount > 1 && pastKey) ? [q, k, pastKey] : [q, k];
-      if (relativePositionBias) {
-        inputsK.push(relativePositionBias);
-      }
+  const inputsK = [q, k];
+  if (parameters.kvNumHeads === undefined && outputCount > 1 && pastKey && ShapeUtil.size(pastKey.dims) > 0) {
+    inputsK.push(pastKey);
+  }
+  if (attentionBias) {
+    inputsK.push(attentionBias);
+  }
 
-      // Run AttentionProbs
-      const probs = context.compute(
-          createAttentionProbsProgramInfo(
-              context, q, k, outputCount > 1 ? pastKey : undefined, relativePositionBias, parameters, attributes,
-              pastSequenceLength),
-          {inputs: inputsK, outputs: (parameters.kvNumHeads === undefined && outputCount > 1) ? [-1, 1] : [-1]})[0];
+  // Run AttentionProbs
+  const probs = context.compute(
+    createAttentionProbsProgramInfo(
+      outputCount,
+      q,
+      k,
+      pastKey,
+      attentionBias,
+      parameters,
+      attributes,
+      pastSequenceLength,
+    ),
+    { inputs: inputsK, outputs: parameters.kvNumHeads === undefined && outputCount > 1 ? [-1, 1] : [-1] },
+  )[0];
 
-      // Run Softmax
-      context.compute(
-          createInPlaceSoftmaxProgramInfo(
-              context, probs, parameters.batchSize * parameters.numHeads * parameters.sequenceLength,
-              totalSequenceLength),
-          {inputs: [probs], outputs: []});
+  // Run Softmax
+  context.compute(
+    createInPlaceSoftmaxProgramInfo(
+      probs,
+      parameters.batchSize * parameters.numHeads * parameters.sequenceLength,
+      totalSequenceLength,
+    ),
+    { inputs: [probs], outputs: [] },
+  );
 
-      // Run AttrionScore
-      const inputsV =
-          (parameters.kvNumHeads === undefined && outputCount > 1 && pastValue) ? [probs, v, pastValue] : [probs, v];
-      context.compute(
-          createVxAttentionScoreProgramInfo(
-              context, probs, v, outputCount > 1 && pastValue ? pastValue : undefined, parameters, pastSequenceLength),
-          {inputs: inputsV, outputs: (parameters.kvNumHeads === undefined && outputCount > 1) ? [0, 2] : [0]});
-    };
+  // Run AttrionScore
+  const inputsV = [probs, v];
+  if (parameters.kvNumHeads === undefined && outputCount > 1 && pastValue && ShapeUtil.size(pastValue.dims) > 0) {
+    inputsV.push(pastValue);
+  }
+  context.compute(createVxAttentionScoreProgramInfo(outputCount, probs, v, pastValue, parameters, pastSequenceLength), {
+    inputs: inputsV,
+    outputs: parameters.kvNumHeads === undefined && outputCount > 1 ? [0, 2] : [0],
+  });
+};
 
 const prepare = (context: ComputeContext, parameters: AttentionParameters) => {
-  const outputShape = [
-    parameters.batchSize,
-    parameters.numHeads,
-    parameters.sequenceLength,
-    parameters.headSize,
-  ];
+  const outputShape = [parameters.batchSize, parameters.numHeads, parameters.sequenceLength, parameters.headSize];
   const M = parameters.sequenceLength;
   const K = parameters.inputHiddenSize;
   const N = parameters.headSize;
@@ -662,14 +733,17 @@ const prepare = (context: ComputeContext, parameters: AttentionParameters) => {
   const dispatch = {
     x: Math.ceil(parameters.headSize / TILE_SIZE),
     y: Math.ceil(parameters.sequenceLength / TILE_SIZE),
-    z: parameters.batchSize * parameters.numHeads
+    z: parameters.batchSize * parameters.numHeads,
   };
   const inputs = [context.inputs[0], context.inputs[1], context.inputs[2]];
   const programUniforms: ProgramUniform[] = [
-    {type: DataType.uint32, data: M}, {type: DataType.uint32, data: K}, {type: DataType.uint32, data: N},
-    {type: DataType.uint32, data: parameters.numHeads}, {type: DataType.uint32, data: parameters.headSize},
-    {type: DataType.uint32, data: parameters.hiddenSize},
-    {type: DataType.uint32, data: parameters.hiddenSize + parameters.hiddenSize + parameters.vHiddenSize}
+    { type: DataType.uint32, data: M },
+    { type: DataType.uint32, data: K },
+    { type: DataType.uint32, data: N },
+    { type: DataType.uint32, data: parameters.numHeads },
+    { type: DataType.uint32, data: parameters.headSize },
+    { type: DataType.uint32, data: parameters.hiddenSize },
+    { type: DataType.uint32, data: parameters.hiddenSize + parameters.hiddenSize + parameters.vHiddenSize },
   ];
 
   const getShaderSource = (shaderHelper: ShaderHelper) => {
@@ -682,8 +756,13 @@ const prepare = (context: ComputeContext, parameters: AttentionParameters) => {
     const dataType = input.type.storage;
 
     const uniforms: UniformsArrayType = [
-      {name: 'M', type: 'u32'}, {name: 'K', type: 'u32'}, {name: 'N', type: 'u32'}, {name: 'num_heads', type: 'u32'},
-      {name: 'head_size', type: 'u32'}, {name: 'hidden_size', type: 'u32'}, {name: 'ldb', type: 'u32'}
+      { name: 'M', type: 'u32' },
+      { name: 'K', type: 'u32' },
+      { name: 'N', type: 'u32' },
+      { name: 'num_heads', type: 'u32' },
+      { name: 'head_size', type: 'u32' },
+      { name: 'hidden_size', type: 'u32' },
+      { name: 'ldb', type: 'u32' },
     ];
     return `
   const TILE_SIZE = ${TILE_SIZE}u;
@@ -692,9 +771,7 @@ const prepare = (context: ComputeContext, parameters: AttentionParameters) => {
   var<workgroup> tileWeightK: array<${dataType}, ${TILE_SIZE * TILE_SIZE}>;
   var<workgroup> tileWeightV: array<${dataType}, ${TILE_SIZE * TILE_SIZE}>;
   ${shaderHelper.registerUniforms(uniforms).declareVariables(input, weight, bias, outputQ, outputK, outputV)}
-  ${shaderHelper.mainStart([
-      TILE_SIZE, TILE_SIZE, 1
-    ])}
+  ${shaderHelper.mainStart([TILE_SIZE, TILE_SIZE, 1])}
     let batchIndex = workgroup_id.z / uniforms.num_heads;
     let headNumber = workgroup_id.z % uniforms.num_heads;
     let m = global_id.y;
@@ -746,21 +823,22 @@ const prepare = (context: ComputeContext, parameters: AttentionParameters) => {
   };
 
   return context.compute(
-      {
-        name: 'AttentionPrepare',
-        shaderCache: {inputDependencies: ['type', 'type', 'type']},
-        getRunData: () => ({
-          outputs: [
-            {dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default},
-            {dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default},
-            {dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default},
-          ],
-          dispatchGroup: dispatch,
-          programUniforms
-        }),
-        getShaderSource,
-      },
-      {inputs, outputs: [-1, -1, -1]});
+    {
+      name: 'AttentionPrepare',
+      shaderCache: { inputDependencies: ['type', 'type', 'type'] },
+      getRunData: () => ({
+        outputs: [
+          { dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default },
+          { dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default },
+          { dims: outputShape, dataType: context.inputs[0].dataType, gpuDataType: GpuDataType.default },
+        ],
+        dispatchGroup: dispatch,
+        programUniforms,
+      }),
+      getShaderSource,
+    },
+    { inputs, outputs: [-1, -1, -1] },
+  );
 };
 
 export const attention = (context: ComputeContext, attributes: AttentionAttrs): void => {
@@ -769,5 +847,16 @@ export const attention = (context: ComputeContext, attributes: AttentionAttrs):
   const [q, k, v] = prepare(context, params);
 
   return applyAttention(
-      context, q, k, v, context.inputs[4], undefined, undefined, undefined, context.inputs[5], params, attributes);
+    context,
+    q,
+    k,
+    v,
+    context.inputs[4],
+    undefined,
+    undefined,
+    undefined,
+    context.inputs[5],
+    params,
+    attributes,
+  );
 };
