@@ -1,12 +1,12 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
 // Licensed under the MIT License.
 
-import {InferenceSession} from 'onnxruntime-common';
+import { InferenceSession } from 'onnxruntime-common';
 
-import {getInstance} from './wasm-factory';
-import {allocWasmString, checkLastError, iterateExtraOptions} from './wasm-utils';
+import { getInstance } from './wasm-factory';
+import { allocWasmString, checkLastError, iterateExtraOptions } from './wasm-utils';
 
-const getGraphOptimzationLevel = (graphOptimizationLevel: string|unknown): number => {
+const getGraphOptimzationLevel = (graphOptimizationLevel: string | unknown): number => {
   switch (graphOptimizationLevel) {
     case 'disabled':
       return 0;
@@ -21,7 +21,7 @@ const getGraphOptimzationLevel = (graphOptimizationLevel: string|unknown): numbe
   }
 };
 
-const getExecutionMode = (executionMode: 'sequential'|'parallel'): number => {
+const getExecutionMode = (executionMode: 'sequential' | 'parallel'): number => {
   switch (executionMode) {
     case 'sequential':
       return 0;
@@ -46,67 +46,68 @@ const appendDefaultOptions = (options: InferenceSession.SessionOptions): void =>
   }
 
   // if using JSEP with WebGPU, always disable memory pattern
-  if (options.executionProviders &&
-      options.executionProviders.some(ep => (typeof ep === 'string' ? ep : ep.name) === 'webgpu')) {
+  if (
+    options.executionProviders &&
+    options.executionProviders.some((ep) => (typeof ep === 'string' ? ep : ep.name) === 'webgpu')
+  ) {
     options.enableMemPattern = false;
   }
 };
 
-const setExecutionProviders =
-    (sessionOptionsHandle: number, executionProviders: readonly InferenceSession.ExecutionProviderConfig[],
-     allocs: number[]): void => {
-      for (const ep of executionProviders) {
-        let epName = typeof ep === 'string' ? ep : ep.name;
+const setExecutionProviders = (
+  sessionOptionsHandle: number,
+  executionProviders: readonly InferenceSession.ExecutionProviderConfig[],
+  allocs: number[],
+): void => {
+  for (const ep of executionProviders) {
+    let epName = typeof ep === 'string' ? ep : ep.name;
 
-        // check EP name
-        switch (epName) {
-          case 'webnn':
-            epName = 'WEBNN';
-            if (typeof ep !== 'string') {
-              const webnnOptions = ep as InferenceSession.WebNNExecutionProviderOption;
-              // const context = (webnnOptions as InferenceSession.WebNNOptionsWithMLContext)?.context;
-              const deviceType = (webnnOptions as InferenceSession.WebNNContextOptions)?.deviceType;
-              if (deviceType) {
-                const keyDataOffset = allocWasmString('deviceType', allocs);
-                const valueDataOffset = allocWasmString(deviceType, allocs);
-                if (getInstance()._OrtAddSessionConfigEntry(sessionOptionsHandle, keyDataOffset, valueDataOffset) !==
-                    0) {
-                  checkLastError(`Can't set a session config entry: 'deviceType' - ${deviceType}.`);
-                }
-              }
+    // check EP name
+    switch (epName) {
+      case 'webnn':
+        epName = 'WEBNN';
+        if (typeof ep !== 'string') {
+          const webnnOptions = ep as InferenceSession.WebNNExecutionProviderOption;
+          // const context = (webnnOptions as InferenceSession.WebNNOptionsWithMLContext)?.context;
+          const deviceType = (webnnOptions as InferenceSession.WebNNContextOptions)?.deviceType;
+          if (deviceType) {
+            const keyDataOffset = allocWasmString('deviceType', allocs);
+            const valueDataOffset = allocWasmString(deviceType, allocs);
+            if (getInstance()._OrtAddSessionConfigEntry(sessionOptionsHandle, keyDataOffset, valueDataOffset) !== 0) {
+              checkLastError(`Can't set a session config entry: 'deviceType' - ${deviceType}.`);
             }
-            break;
-          case 'webgpu':
-            epName = 'JS';
-            if (typeof ep !== 'string') {
-              const webgpuOptions = ep as InferenceSession.WebGpuExecutionProviderOption;
-              if (webgpuOptions?.preferredLayout) {
-                if (webgpuOptions.preferredLayout !== 'NCHW' && webgpuOptions.preferredLayout !== 'NHWC') {
-                  throw new Error(`preferredLayout must be either 'NCHW' or 'NHWC': ${webgpuOptions.preferredLayout}`);
-                }
-                const keyDataOffset = allocWasmString('preferredLayout', allocs);
-                const valueDataOffset = allocWasmString(webgpuOptions.preferredLayout, allocs);
-                if (getInstance()._OrtAddSessionConfigEntry(sessionOptionsHandle, keyDataOffset, valueDataOffset) !==
-                    0) {
-                  checkLastError(
-                      `Can't set a session config entry: 'preferredLayout' - ${webgpuOptions.preferredLayout}.`);
-                }
-              }
-            }
-            break;
-          case 'wasm':
-          case 'cpu':
-            continue;
-          default:
-            throw new Error(`not supported execution provider: ${epName}`);
+          }
         }
+        break;
+      case 'webgpu':
+        epName = 'JS';
+        if (typeof ep !== 'string') {
+          const webgpuOptions = ep as InferenceSession.WebGpuExecutionProviderOption;
+          if (webgpuOptions?.preferredLayout) {
+            if (webgpuOptions.preferredLayout !== 'NCHW' && webgpuOptions.preferredLayout !== 'NHWC') {
+              throw new Error(`preferredLayout must be either 'NCHW' or 'NHWC': ${webgpuOptions.preferredLayout}`);
+            }
+            const keyDataOffset = allocWasmString('preferredLayout', allocs);
+            const valueDataOffset = allocWasmString(webgpuOptions.preferredLayout, allocs);
+            if (getInstance()._OrtAddSessionConfigEntry(sessionOptionsHandle, keyDataOffset, valueDataOffset) !== 0) {
+              checkLastError(`Can't set a session config entry: 'preferredLayout' - ${webgpuOptions.preferredLayout}.`);
+            }
+          }
+        }
+        break;
+      case 'wasm':
+      case 'cpu':
+        continue;
+      default:
+        throw new Error(`not supported execution provider: ${epName}`);
+    }
 
-        const epNameDataOffset = allocWasmString(epName, allocs);
-        if (getInstance()._OrtAppendExecutionProvider(sessionOptionsHandle, epNameDataOffset) !== 0) {
-          checkLastError(`Can't append execution provider: ${epName}.`);
-        }
-      }
-    };
+    const epNameDataOffset = allocWasmString(epName, allocs);
+    if (getInstance()._OrtAppendExecutionProvider(sessionOptionsHandle, epNameDataOffset) !== 0) {
+      checkLastError(`Can't append execution provider: ${epName}.`);
+    }
+  }
+};
 
 export const setSessionOptions = (options?: InferenceSession.SessionOptions): [number, number[]] => {
   const wasm = getInstance();
@@ -120,28 +121,37 @@ export const setSessionOptions = (options?: InferenceSession.SessionOptions): [n
     const graphOptimizationLevel = getGraphOptimzationLevel(sessionOptions.graphOptimizationLevel ?? 'all');
     const executionMode = getExecutionMode(sessionOptions.executionMode ?? 'sequential');
     const logIdDataOffset =
-        typeof sessionOptions.logId === 'string' ? allocWasmString(sessionOptions.logId, allocs) : 0;
+      typeof sessionOptions.logId === 'string' ? allocWasmString(sessionOptions.logId, allocs) : 0;
 
-    const logSeverityLevel = sessionOptions.logSeverityLevel ?? 2;  // Default to 2 - warning
+    const logSeverityLevel = sessionOptions.logSeverityLevel ?? 2; // Default to 2 - warning
     if (!Number.isInteger(logSeverityLevel) || logSeverityLevel < 0 || logSeverityLevel > 4) {
       throw new Error(`log serverity level is not valid: ${logSeverityLevel}`);
     }
 
-    const logVerbosityLevel = sessionOptions.logVerbosityLevel ?? 0;  // Default to 0 - verbose
+    const logVerbosityLevel = sessionOptions.logVerbosityLevel ?? 0; // Default to 0 - verbose
     if (!Number.isInteger(logVerbosityLevel) || logVerbosityLevel < 0 || logVerbosityLevel > 4) {
       throw new Error(`log verbosity level is not valid: ${logVerbosityLevel}`);
     }
 
-    const optimizedModelFilePathOffset = typeof sessionOptions.optimizedModelFilePath === 'string' ?
-        allocWasmString(sessionOptions.optimizedModelFilePath, allocs) :
-        0;
+    const optimizedModelFilePathOffset =
+      typeof sessionOptions.optimizedModelFilePath === 'string'
+        ? allocWasmString(sessionOptions.optimizedModelFilePath, allocs)
+        : 0;
 
     sessionOptionsHandle = wasm._OrtCreateSessionOptions(
-        graphOptimizationLevel, !!sessionOptions.enableCpuMemArena, !!sessionOptions.enableMemPattern, executionMode,
-        !!sessionOptions.enableProfiling, 0, logIdDataOffset, logSeverityLevel, logVerbosityLevel,
-        optimizedModelFilePathOffset);
+      graphOptimizationLevel,
+      !!sessionOptions.enableCpuMemArena,
+      !!sessionOptions.enableMemPattern,
+      executionMode,
+      !!sessionOptions.enableProfiling,
+      0,
+      logIdDataOffset,
+      logSeverityLevel,
+      logVerbosityLevel,
+      optimizedModelFilePathOffset,
+    );
     if (sessionOptionsHandle === 0) {
-      checkLastError('Can\'t create session options.');
+      checkLastError("Can't create session options.");
     }
 
     if (sessionOptions.executionProviders) {
@@ -156,7 +166,8 @@ export const setSessionOptions = (options?: InferenceSession.SessionOptions): [n
       const valueDataOffset = allocWasmString(sessionOptions.enableGraphCapture.toString(), allocs);
       if (wasm._OrtAddSessionConfigEntry(sessionOptionsHandle, keyDataOffset, valueDataOffset) !== 0) {
         checkLastError(
-            `Can't set a session config entry: 'enableGraphCapture' - ${sessionOptions.enableGraphCapture}.`);
+          `Can't set a session config entry: 'enableGraphCapture' - ${sessionOptions.enableGraphCapture}.`,
+        );
       }
     }
 
@@ -191,7 +202,7 @@ export const setSessionOptions = (options?: InferenceSession.SessionOptions): [n
     if (sessionOptionsHandle !== 0) {
       wasm._OrtReleaseSessionOptions(sessionOptionsHandle);
     }
-    allocs.forEach(alloc => wasm._free(alloc));
+    allocs.forEach((alloc) => wasm._free(alloc));
     throw e;
   }
 };
