@@ -5,11 +5,11 @@
 
 const path = require('path');
 const fs = require('fs-extra');
-const {spawn} = require('child_process');
+const { spawn } = require('child_process');
 const startServer = require('./simple-http-server');
 const minimist = require('minimist');
 
-const {NODEJS_TEST_CASES, BROWSER_TEST_CASES, BUNDLER_TEST_CASES} = require('./run-data');
+const { NODEJS_TEST_CASES, BROWSER_TEST_CASES, BUNDLER_TEST_CASES } = require('./run-data');
 
 // copy whole folder to out-side of <ORT_ROOT>/js/ because we need to test in a folder that no `package.json` file
 // exists in its parent folder.
@@ -28,7 +28,7 @@ fs.copySync(TEST_E2E_SRC_FOLDER, TEST_E2E_RUN_FOLDER);
 // always use a new folder as user-data-dir
 let nextUserDataDirId = 0;
 function getNextUserDataDir() {
-  const dir = path.resolve(CHROME_USER_DATA_FOLDER, nextUserDataDirId.toString())
+  const dir = path.resolve(CHROME_USER_DATA_FOLDER, nextUserDataDirId.toString());
   nextUserDataDirId++;
   fs.emptyDirSync(dir);
   return dir;
@@ -39,10 +39,10 @@ const BROWSER = minimist(process.argv.slice(2)).browser || 'Chrome_default';
 
 async function main() {
   // find packed package
-  const {globbySync} = await import('globby');
+  const { globbySync } = await import('globby');
 
   const ORT_COMMON_FOLDER = path.resolve(JS_ROOT_FOLDER, 'common');
-  const ORT_COMMON_PACKED_FILEPATH_CANDIDATES = globbySync('onnxruntime-common-*.tgz', {cwd: ORT_COMMON_FOLDER});
+  const ORT_COMMON_PACKED_FILEPATH_CANDIDATES = globbySync('onnxruntime-common-*.tgz', { cwd: ORT_COMMON_FOLDER });
 
   const PACKAGES_TO_INSTALL = [];
 
@@ -53,7 +53,7 @@ async function main() {
   }
 
   const ORT_WEB_FOLDER = path.resolve(JS_ROOT_FOLDER, 'web');
-  const ORT_WEB_PACKED_FILEPATH_CANDIDATES = globbySync('onnxruntime-web-*.tgz', {cwd: ORT_WEB_FOLDER});
+  const ORT_WEB_PACKED_FILEPATH_CANDIDATES = globbySync('onnxruntime-web-*.tgz', { cwd: ORT_WEB_FOLDER });
   if (ORT_WEB_PACKED_FILEPATH_CANDIDATES.length !== 1) {
     throw new Error('cannot find exactly single package for onnxruntime-web.');
   }
@@ -65,7 +65,7 @@ async function main() {
   await runInShell(`npm install`);
 
   // npm install with "--cache" to install packed packages with an empty cache folder
-  await runInShell(`npm install --cache "${NPM_CACHE_FOLDER}" ${PACKAGES_TO_INSTALL.map(i => `"${i}"`).join(' ')}`);
+  await runInShell(`npm install --cache "${NPM_CACHE_FOLDER}" ${PACKAGES_TO_INSTALL.map((i) => `"${i}"`).join(' ')}`);
 
   // prepare .wasm files for path override testing
   prepareWasmPathOverrideFiles();
@@ -78,11 +78,15 @@ async function main() {
   prepareEsmLoaderFiles();
 
   await fs.symlink(
-      path.resolve(TEST_E2E_RUN_FOLDER, 'node_modules', 'onnxruntime-web', 'dist'), path.join(serverWwwRoot, 'dist'),
-      'junction');
+    path.resolve(TEST_E2E_RUN_FOLDER, 'node_modules', 'onnxruntime-web', 'dist'),
+    path.join(serverWwwRoot, 'dist'),
+    'junction',
+  );
   await fs.symlink(
-      path.resolve(TEST_E2E_RUN_FOLDER, 'test-wasm-path-override'), path.join(serverWwwRoot, 'test-wasm-path-override'),
-      'junction');
+    path.resolve(TEST_E2E_RUN_FOLDER, 'test-wasm-path-override'),
+    path.join(serverWwwRoot, 'test-wasm-path-override'),
+    'junction',
+  );
 
   // start a HTTP server for hosting .wasm files (for cross-origin testing)
   const server = startServer(serverWwwRoot, 8081);
@@ -94,17 +98,16 @@ async function main() {
     await testAllNodejsCases();
 
     // test cases with self-host (ort hosted in same origin)
-    await testAllBrowserCases({hostInKarma: true});
+    await testAllBrowserCases({ hostInKarma: true });
 
     // test cases without self-host (ort hosted in different origin)
-    await testAllBrowserCases({hostInKarma: false});
+    await testAllBrowserCases({ hostInKarma: false });
 
     // run bundlers
     await runInShell(`npm run build`);
 
     // test package consuming test
     await testAllBrowserPackagesConsumingCases();
-
   } finally {
     // close the server after all tests
     await server.close();
@@ -112,25 +115,32 @@ async function main() {
 }
 
 function prepareEsmLoaderFiles() {
-  const allEsmFiles = [...new Set(BROWSER_TEST_CASES.map(i => i[3]).filter(i => i && i.endsWith('.mjs')))];
+  const allEsmFiles = [...new Set(BROWSER_TEST_CASES.map((i) => i[3]).filter((i) => i && i.endsWith('.mjs')))];
 
   // self-hosted
   fs.emptyDirSync(path.join(TEST_E2E_RUN_FOLDER, 'esm-loaders'));
   fs.emptyDirSync(path.join(TEST_E2E_RUN_FOLDER, 'wwwroot', 'esm-loaders'));
-  allEsmFiles.forEach(i => {
+  allEsmFiles.forEach((i) => {
     fs.writeFileSync(
-        path.join(TEST_E2E_RUN_FOLDER, 'esm-loaders', i),
-        `import * as x from '../node_modules/onnxruntime-web/dist/${i}'; globalThis.ort = x;`);
+      path.join(TEST_E2E_RUN_FOLDER, 'esm-loaders', i),
+      `import * as x from '../node_modules/onnxruntime-web/dist/${i}'; globalThis.ort = x;`,
+    );
     fs.writeFileSync(
-        path.join(TEST_E2E_RUN_FOLDER, 'wwwroot', 'esm-loaders', i),
-        `import * as x from '../dist/${i}'; globalThis.ort = x;`);
+      path.join(TEST_E2E_RUN_FOLDER, 'wwwroot', 'esm-loaders', i),
+      `import * as x from '../dist/${i}'; globalThis.ort = x;`,
+    );
   });
 }
 
 function prepareWasmPathOverrideFiles() {
   const folder = path.join(TEST_E2E_RUN_FOLDER, 'test-wasm-path-override');
-  const sourceFile =
-      path.join(TEST_E2E_RUN_FOLDER, 'node_modules', 'onnxruntime-web', 'dist', 'ort-wasm-simd-threaded');
+  const sourceFile = path.join(
+    TEST_E2E_RUN_FOLDER,
+    'node_modules',
+    'onnxruntime-web',
+    'dist',
+    'ort-wasm-simd-threaded',
+  );
   fs.emptyDirSync(folder);
   fs.copyFileSync(`${sourceFile}.mjs`, path.join(folder, 'ort-wasm-simd-threaded.mjs'));
   fs.copyFileSync(`${sourceFile}.wasm`, path.join(folder, 'ort-wasm-simd-threaded.wasm'));
@@ -144,23 +154,23 @@ async function testAllNodejsCases() {
   }
 }
 
-async function testAllBrowserCases({hostInKarma}) {
+async function testAllBrowserCases({ hostInKarma }) {
   for (const [testForSameOrigin, testForCrossOrigin, main, ortMain, args] of BROWSER_TEST_CASES) {
     if (hostInKarma && testForSameOrigin) {
-      await runKarma({hostInKarma, main, ortMain, args});
-      await runKarma({hostInKarma, main, ortMain, args, enableSharedArrayBuffer: true});
+      await runKarma({ hostInKarma, main, ortMain, args });
+      await runKarma({ hostInKarma, main, ortMain, args, enableSharedArrayBuffer: true });
     }
     if (!hostInKarma && testForCrossOrigin) {
-      await runKarma({hostInKarma, main, ortMain, args});
-      await runKarma({hostInKarma, main, ortMain, args, enableSharedArrayBuffer: true});
+      await runKarma({ hostInKarma, main, ortMain, args });
+      await runKarma({ hostInKarma, main, ortMain, args, enableSharedArrayBuffer: true });
     }
   }
 }
 
 async function testAllBrowserPackagesConsumingCases() {
   for (const [main, format] of BUNDLER_TEST_CASES) {
-    await runKarma({hostInKarma: true, main, ortMain: '', format});
-    await runKarma({hostInKarma: true, main, ortMain: '', format, enableSharedArrayBuffer: true});
+    await runKarma({ hostInKarma: true, main, ortMain: '', format });
+    await runKarma({ hostInKarma: true, main, ortMain: '', format, enableSharedArrayBuffer: true });
   }
 }
 
@@ -171,15 +181,17 @@ async function runKarma({
   ortMain = 'ort.min.js',
   format = 'iife',
   enableSharedArrayBuffer = false,
-  args = []
+  args = [],
 }) {
   const selfHostFlag = hostInKarma ? '--self-host' : '';
-  const argsStr = args.map(i => `--test-args=${i}`).join(' ');
+  const argsStr = args.map((i) => `--test-args=${i}`).join(' ');
   const formatFlag = `--format=${format}`;
   const enableSharedArrayBufferFlag = enableSharedArrayBuffer ? '--enable-shared-array-buffer' : '';
   await runInShell(
-      `npx karma start --single-run --browsers ${browser} ${selfHostFlag} --ort-main=${ortMain} --test-main=${
-          main} --user-data=${getNextUserDataDir()} ${argsStr} ${formatFlag} ${enableSharedArrayBufferFlag}`);
+    `npx karma start --single-run --browsers ${browser} ${selfHostFlag} --ort-main=${ortMain} --test-main=${
+      main
+    } --user-data=${getNextUserDataDir()} ${argsStr} ${formatFlag} ${enableSharedArrayBufferFlag}`,
+  );
 }
 
 async function runInShell(cmd) {
@@ -188,8 +200,8 @@ async function runInShell(cmd) {
   console.log(' > ' + cmd);
   console.log('===============================================================');
   let complete = false;
-  const childProcess = spawn(cmd, {shell: true, stdio: 'inherit', cwd: TEST_E2E_RUN_FOLDER});
-  childProcess.on('close', function(code) {
+  const childProcess = spawn(cmd, { shell: true, stdio: 'inherit', cwd: TEST_E2E_RUN_FOLDER });
+  childProcess.on('close', function (code) {
     if (code !== 0) {
       process.exit(code);
     } else {
@@ -202,8 +214,8 @@ async function runInShell(cmd) {
 }
 
 async function delay(ms) {
-  return new Promise(function(resolve) {
-    setTimeout(function() {
+  return new Promise(function (resolve) {
+    setTimeout(function () {
       resolve();
     }, ms);
   });
