@@ -23,6 +23,7 @@ class SessionState;
 class GraphViewer;
 class OrtValueNameIdxMap;
 class DataTransferManager;
+class ExternalDataLoaderManager;
 class NodeArg;
 #if !defined(ORT_MINIMAL_BUILD) && defined(ORT_MEMORY_PROFILE)
 class MemoryInfo;
@@ -45,11 +46,33 @@ common::Status SaveInitializedTensors(
     const SaveTensorFunction& save_tensor_func,
     const logging::Logger& logger,
     const DataTransferManager& data_transfer_mgr,
+    const ExternalDataLoaderManager& external_data_loader_mgr,
     const ExecutionPlanBase& exec_plan,
     const SessionOptions& session_options,
     const MemoryProfileFunction& memory_profile_func,
     std::unordered_map<std::string, std::unique_ptr<Tensor>>& buffered_tensors);
 
+common::Status AllocateTensor(
+    const onnxruntime::MemBuffer* m,
+    std::unique_ptr<onnxruntime::Tensor>& p_tensor,
+    const onnxruntime::DataTypeImpl* const& type,
+    onnxruntime::TensorShape& tensor_shape,
+    bool use_device_allocator_for_initializers,
+    const onnxruntime::AllocatorPtr& alloc);
+
+common::Status AllocateTensorOnDeviceOrMemory(
+    bool use_device_allocator_for_initializers,
+    onnxruntime::TensorShape& tensor_shape,
+    const onnxruntime::DataTypeImpl* const& type,
+    const onnxruntime::AllocatorPtr& alloc,
+    std::unique_ptr<onnxruntime::Tensor>& p_tensor);
+
+common::Status CopyTensorFromCPUToDevice(
+    const onnxruntime::DataTransferManager& data_transfer_mgr,
+    std::unique_ptr<onnxruntime::Tensor>& p_deserialize_tensor,
+    std::unique_ptr<onnxruntime::Tensor>& p_tensor,
+    OrtValue& ort_value);
+
 common::Status SaveInputOutputNamesToNodeMapping(const GraphViewer& graph,
                                                  SessionState& session_state,
                                                  gsl::span<const NodeArg* const> implicit_inputs);
