@@ -71,6 +71,7 @@ MLAS_INTERNAL_DATA const float MlasMinimumF32Value = std::numeric_limits<float>:
 struct MLAS_SOFTMAX_WORK_BLOCK {
     ptrdiff_t ThreadCountN;
     bool LogSoftmax;
+    bool SmoothSoftmax;
     const float* Input;
     float* Output;
     size_t N;
@@ -81,7 +82,7 @@ MLAS_FORCEINLINE
 MLAS_FLOAT32X4
 MlasComputeExpVector(
     MLAS_FLOAT32X4 Vector
-    )
+)
 /*++
 
 Routine Description:
@@ -148,6 +149,9 @@ Return Value:
     // instead.
     normal = _mm_min_epi16(normal, MaximumExponent);
     normal = _mm_max_epi16(normal, MinimumExponent);
+#elif defined(MLAS_LSX_INTRINSICS)
+    normal = __lsx_vmin_h(normal, MaximumExponent);
+    normal = __lsx_vmax_h(normal, MinimumExponent);
 #else
     normal = MlasMinimumInt32x4(normal, MaximumExponent);
     normal = MlasMaximumInt32x4(normal, MinimumExponent);
@@ -183,7 +187,7 @@ MlasComputeExpF32Kernel(
     const float* Input,
     float* Output,
     size_t N
-    )
+)
 /*++
 
 Routine Description:
@@ -205,7 +209,6 @@ Return Value:
 --*/
 {
     while (N > 0) {
-
         MLAS_FLOAT32X4 Vector;
 
         if (N >= 4) {
@@ -215,6 +218,8 @@ Return Value:
             // N.B. SSE2 lacks a broadcast load instruction, so avoid a shuffle
             // and use zeroes for the upper elements.
             Vector = _mm_load_ss(Input);
+#elif defined(MLAS_LSX_INTRINSICS)
+            Vector = (MLAS_FLOAT32X4)__lsx_vldrepl_w(Input, 0);
 #else
             Vector = MlasBroadcastFloat32x4(Input);
 #endif
@@ -223,7 +228,6 @@ Return Value:
         Vector = MlasComputeExpVector(Vector);
 
         if (N >= 4) {
-
             MlasStoreFloat32x4(Output, Vector);
 
             Input += 4;
@@ -231,7 +235,6 @@ Return Value:
             N -= 4;
 
         } else {
-
             MlasStoreLaneFloat32x4<0>(Output, Vector);
 
             Input += 1;
@@ -247,7 +250,7 @@ MlasComputeExp(
     const float* Input,
     float* Output,
     size_t N
-    )
+)
 /*++
 
 Routine Description:
@@ -282,7 +285,7 @@ MLAS_FLOAT32X4
 MlasComputeSumExpVector(
     MLAS_FLOAT32X4 Vector,
     MLAS_FLOAT32X4 NegativeMaximumVector
-    )
+)
 /*++
 
 Routine Description:
@@ -374,7 +377,7 @@ MlasComputeSumExpF32Kernel(
     float* Output,
     size_t N,
     const float* NegativeMaximum
-    )
+)
 /*++
 
 Routine Description:
@@ -406,7 +409,6 @@ Return Value:
     float Accumulator = 0.0f;
 
     if (N >= 4) {
-
         MLAS_FLOAT32X4 AccumulatorVector = MlasZeroFloat32x4();
 
 #if !defined(MLAS_SSE2_INTRINSICS)
@@ -421,7 +423,6 @@ Return Value:
         //
 
         while (N >= 8) {
-
             MLAS_FLOAT32X4 Vector0 = MlasLoadFloat32x4(Input);
             MLAS_FLOAT32X4 Vector1 = MlasLoadFloat32x4(Input + 4);
 
@@ -443,7 +444,6 @@ Return Value:
 #endif
 
         while (N >= 4) {
-
             MLAS_FLOAT32X4 Vector = MlasLoadFloat32x4(Input);
 
             Vector = MlasComputeSumExpVector(Vector, NegativeMaximumVector);
@@ -462,11 +462,12 @@ Return Value:
     }
 
     while (N > 0) {
-
 #if defined(MLAS_SSE2_INTRINSICS)
         // N.B. SSE2 lacks a broadcast load instruction, so avoid a shuffle and
         // use zeroes for the upper elements.
         MLAS_FLOAT32X4 Vector = _mm_load_ss(Input);
+#elif defined(MLAS_LSX_INTRINSICS)
+        MLAS_FLOAT32X4 Vector = (MLAS_FLOAT32X4)__lsx_vldrepl_w(Input, 0);
 #else
         MLAS_FLOAT32X4 Vector = MlasBroadcastFloat32x4(Input);
 #endif
@@ -491,7 +492,7 @@ MLASCALL
 MlasReduceMaximumF32Kernel(
     const float* Input,
     size_t N
-    )
+)
 /*++
 
 Routine Description:
@@ -514,17 +515,14 @@ Return Value:
     float Maximum = MlasMinimumF32Value;
 
     if (N >= 4) {
-
         MLAS_FLOAT32X4 MaximumVector0 = MlasBroadcastFloat32x4(Maximum);
 
         if (N >= 16) {
-
             MLAS_FLOAT32X4 MaximumVector1 = MaximumVector0;
             MLAS_FLOAT32X4 MaximumVector2 = MaximumVector0;
             MLAS_FLOAT32X4 MaximumVector3 = MaximumVector0;
 
             while (N >= 16) {
-
                 MaximumVector0 = MlasMaximumFloat32x4(MaximumVector0, MlasLoadFloat32x4(Input));
                 MaximumVector1 = MlasMaximumFloat32x4(MaximumVector1, MlasLoadFloat32x4(Input + 4));
                 MaximumVector2 = MlasMaximumFloat32x4(MaximumVector2, MlasLoadFloat32x4(Input + 8));
@@ -540,7 +538,6 @@ Return Value:
         }
 
         while (N >= 4) {
-
             MaximumVector0 = MlasMaximumFloat32x4(MaximumVector0, MlasLoadFloat32x4(Input));
 
             Input += 4;
@@ -551,7 +548,6 @@ Return Value:
     }
 
     while (N > 0) {
-
         Maximum = std::max(Maximum, *Input);
 
         Input += 1;
@@ -568,18 +564,16 @@ MlasReduceMinimumMaximumF32Kernel(
     float* Min,
     float* Max,
     size_t N
-    )
+)
 {
     float tmp_min = std::numeric_limits<float>::max();
     float tmp_max = std::numeric_limits<float>::lowest();
 
     if (N >= 4) {
-
         MLAS_FLOAT32X4 MaximumVector0 = MlasBroadcastFloat32x4(tmp_max);
         MLAS_FLOAT32X4 MinimumVector0 = MlasBroadcastFloat32x4(tmp_min);
 
         if (N >= 16) {
-
             MLAS_FLOAT32X4 MaximumVector1 = MaximumVector0;
             MLAS_FLOAT32X4 MaximumVector2 = MaximumVector0;
             MLAS_FLOAT32X4 MaximumVector3 = MaximumVector0;
@@ -589,7 +583,6 @@ MlasReduceMinimumMaximumF32Kernel(
             MLAS_FLOAT32X4 MinimumVector3 = MinimumVector0;
 
             while (N >= 16) {
-
                 MLAS_FLOAT32X4 InputVector0 = MlasLoadFloat32x4(Input);
                 MLAS_FLOAT32X4 InputVector1 = MlasLoadFloat32x4(Input + 4);
                 MLAS_FLOAT32X4 InputVector2 = MlasLoadFloat32x4(Input + 8);
@@ -619,7 +612,6 @@ MlasReduceMinimumMaximumF32Kernel(
         }
 
         while (N >= 4) {
-
             MLAS_FLOAT32X4 InputVector0 = MlasLoadFloat32x4(Input);
             MaximumVector0 = MlasMaximumFloat32x4(MaximumVector0, InputVector0);
 
@@ -634,7 +626,6 @@ MlasReduceMinimumMaximumF32Kernel(
     }
 
     while (N > 0) {
-
         tmp_max = std::max(tmp_max, *Input);
         tmp_min = std::min(tmp_min, *Input);
 
@@ -652,7 +643,7 @@ MlasComputeSoftmaxOutputF32Kernel(
     float* Output,
     size_t N,
     const float* Parameters
-    )
+)
 /*++
 
 Routine Description:
@@ -679,7 +670,6 @@ Return Value:
     const MLAS_FLOAT32X4 ScaleVector = MlasBroadcastFloat32x4(Scale);
 
     while (N >= 16) {
-
         MLAS_FLOAT32X4 Vector0 = MlasMultiplyFloat32x4(ScaleVector, MlasLoadFloat32x4(Output));
         MLAS_FLOAT32X4 Vector1 = MlasMultiplyFloat32x4(ScaleVector, MlasLoadFloat32x4(Output + 4));
         MLAS_FLOAT32X4 Vector2 = MlasMultiplyFloat32x4(ScaleVector, MlasLoadFloat32x4(Output + 8));
@@ -695,7 +685,6 @@ Return Value:
     }
 
     while (N >= 4) {
-
         MlasStoreFloat32x4(Output, MlasMultiplyFloat32x4(ScaleVector, MlasLoadFloat32x4(Output)));
 
         Output += 4;
@@ -703,7 +692,6 @@ Return Value:
     }
 
     while (N > 0) {
-
         *Output *= Scale;
 
         Output += 1;
@@ -718,7 +706,7 @@ MlasComputeLogSoftmaxOutputF32Kernel(
     float* Output,
     size_t N,
     const float* Parameters
-    )
+)
 /*++
 
 Routine Description:
@@ -750,7 +738,6 @@ Return Value:
     const MLAS_FLOAT32X4 LogarithmVector = MlasBroadcastFloat32x4(Logarithm);
 
     while (N >= 16) {
-
         MLAS_FLOAT32X4 Vector0 = MlasLoadFloat32x4(Input);
         MLAS_FLOAT32X4 Vector1 = MlasLoadFloat32x4(Input + 4);
         MLAS_FLOAT32X4 Vector2 = MlasLoadFloat32x4(Input + 8);
@@ -777,7 +764,6 @@ Return Value:
     }
 
     while (N >= 4) {
-
         MLAS_FLOAT32X4 Vector = MlasLoadFloat32x4(Input);
         Vector = MlasAddFloat32x4(Vector, NegativeMaximumVector);
         Vector = MlasSubtractFloat32x4(Vector, LogarithmVector);
@@ -789,7 +775,6 @@ Return Value:
     }
 
     while (N > 0) {
-
         *Output = *Input + NegativeMaximum - Logarithm;
 
         Input += 1;
@@ -802,7 +787,7 @@ void
 MlasComputeSoftmaxThreaded(
     void* Context,
     ptrdiff_t Index
-    )
+)
 /*++
 
 Routine Description:
@@ -839,67 +824,76 @@ Return Value:
 
     const size_t D = WorkBlock->D;
     const bool LogSoftmax = WorkBlock->LogSoftmax;
+    const bool SmoothSoftmax = WorkBlock->SmoothSoftmax;
 
     const float* Input = WorkBlock->Input + n * D;
     float* Output = WorkBlock->Output + n * D;
 
+#if defined(MLAS_SSE2_INTRINSICS)
+    // TODO: Use std::hardware_constructive_interference_size
+    constexpr size_t CacheLineSize = 64;
+    constexpr size_t ElementsPerCacheLine = CacheLineSize / sizeof(float);
+#endif
+
     while (CountN > 0) {
+#if defined(MLAS_SSE2_INTRINSICS)
+        //
+        // Prefetch the next row of the input buffer.
+        //
+
+        for (size_t i = 0; i * ElementsPerCacheLine < D; i++) {
+            _mm_prefetch((char*)(Input + D) + i * CacheLineSize, _MM_HINT_T0);
+        }
+#endif
 
         //
         // Find the maximum value for the row.
         //
 
-#if defined(MLAS_TARGET_AMD64)
+#if defined(MLAS_TARGET_AMD64) || defined(MLAS_TARGET_LARCH64)
         float Maximum = GetMlasPlatform().ReduceMaximumF32Kernel(Input, D);
 #else
         float Maximum = MlasReduceMaximumF32Kernel(Input, D);
 #endif
         float NegativeMaximum = -Maximum;
+        if (SmoothSoftmax && NegativeMaximum > 0.0f) {
+            NegativeMaximum = 0.0f;
+        }
 
-        if (LogSoftmax) {
-
-            //
-            // Compute the sum of the exponential functions for the row.
-            //
-
+        //
+        // Compute the exponential function for each element of the row (save to Temp if provided) and
+        // compute the sum of these exponential functions.
+        //
+        float* Temp = LogSoftmax ? nullptr : Output;
 #if defined(MLAS_TARGET_AMD64)
-            float Accumulation = GetMlasPlatform().ComputeSumExpF32Kernel(Input, nullptr, D, &NegativeMaximum);
+        float Accumulation = GetMlasPlatform().ComputeSumExpF32Kernel(Input, Temp, D, &NegativeMaximum);
 #else
-            float Accumulation = MlasComputeSumExpF32Kernel(Input, nullptr, D, &NegativeMaximum);
+        float Accumulation = MlasComputeSumExpF32Kernel(Input, Temp, D, &NegativeMaximum);
 #endif
 
+        if (SmoothSoftmax) {
+            Accumulation += expf(NegativeMaximum);
+        }
+
+        if (LogSoftmax) {
             //
             // Compute the log softmax output.
             //
+            float Parameters[] = {NegativeMaximum, std::log(Accumulation)};
 
-            float Parameters[] = { NegativeMaximum, std::log(Accumulation)};
-
-#if defined(MLAS_TARGET_AMD64)
+#if defined(MLAS_TARGET_AMD64) || defined(MLAS_TARGET_LARCH64)
             GetMlasPlatform().ComputeLogSoftmaxOutputF32Kernel(Input, Output, D, Parameters);
 #else
             MlasComputeLogSoftmaxOutputF32Kernel(Input, Output, D, Parameters);
 #endif
 
         } else {
-
-            //
-            // Compute the exponential function for each element of the row and
-            // compute the sum of these exponential functions.
-            //
-
-#if defined(MLAS_TARGET_AMD64)
-            float Accumulation = GetMlasPlatform().ComputeSumExpF32Kernel(Input, Output, D, &NegativeMaximum);
-#else
-            float Accumulation = MlasComputeSumExpF32Kernel(Input, Output, D, &NegativeMaximum);
-#endif
-
             //
             // Normalize the softmax output.
             //
+            float Parameters[] = {1.0f / Accumulation};
 
-            float Parameters[] = { 1.0f / Accumulation };
-
-#if defined(MLAS_TARGET_AMD64)
+#if defined(MLAS_TARGET_AMD64) || defined(MLAS_TARGET_LARCH64)
             GetMlasPlatform().ComputeSoftmaxOutputF32Kernel(Output, D, Parameters);
 #else
             MlasComputeSoftmaxOutputF32Kernel(Output, D, Parameters);
@@ -920,8 +914,9 @@ MlasComputeSoftmax(
     size_t N,
     size_t D,
     bool LogSoftmax,
+    bool SmoothSoftmax,
     MLAS_THREADPOOL* ThreadPool
-    )
+)
 /*++
 
 Routine Description:
@@ -943,6 +938,8 @@ Arguments:
     LogSoftmax - Supplies true if this is a log softmax operation, else false
         if this is a softmax operation.
 
+    SmoothSoftmax - Supplies true if a smooth factor is used in softmax operation.
+
     ThreadPool - Supplies the thread pool object to use, else nullptr if the
         base library threading support should be used.
 
@@ -959,6 +956,7 @@ Return Value:
     //
 
     WorkBlock.LogSoftmax = LogSoftmax;
+    WorkBlock.SmoothSoftmax = SmoothSoftmax;
     WorkBlock.Input = Input;
     WorkBlock.Output = Output;
     WorkBlock.N = N;
