@@ -1,4 +1,5 @@
 // Copyright (c) Microsoft Corporation. All rights reserved.
+// SPDX-FileCopyrightText: Copyright 2024 Arm Limited and/or its affiliates <open-source-office@arm.com>
 // Licensed under the MIT License.
 
 #include <memory>
@@ -23,7 +24,10 @@ void SplitQDQRules(SelectorActionRegistry& qdq_selector_action_registry) {
   const std::string action_name{"dropSplitQDQ"};
   std::unique_ptr<Action> action = std::make_unique<QDQ::SplitReplaceWithQuant>();
 #if !defined(ORT_MINIMAL_BUILD)
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::SplitSelector>(true /*req_equal_quant_params*/);
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::SplitSelector>(true /*req_equal_quant_params*/,
+                                                                                false,
+                                                                                providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"Split", {}}},
                                                          std::move(selector),
@@ -63,25 +67,38 @@ void DropQDQNodesRules(SelectorActionRegistry& qdq_selector_action_registry) {
   //
   // And cannot eliminate the QDQ for MaxPool if the scale is not positive, as a negative
   // scale will change the ordering of the elements between quantized & de-quantized values.
-  std::unique_ptr<NodeSelector> selector_no_16bit = std::make_unique<QDQ::DropQDQNodesSelector>(false);
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector_no_16bit = std::make_unique<QDQ::DropQDQNodesSelector>(false,
+                                                                                                false,
+                                                                                                true,
+                                                                                                providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(drop_action_no_int16_name,
                                                          {{"Resize", {}}},
                                                          std::move(selector_no_16bit),
                                                          std::move(drop_action_no_int16));
 
   std::unique_ptr<NodeSelector> selector_no_16bit_and_positive_scale =
-      std::make_unique<QDQ::DropQDQNodesSelector>(false, true, false);
+      std::make_unique<QDQ::DropQDQNodesSelector>(false, true, false, providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(drop_action_no_int16_and_positive_scale_name,
-                                                         {{"MaxPool", {12}}},
+                                                         {{"MaxPool", {12}},
+                                                          {"ReduceMax", {}},
+                                                          {"ReduceMin", {}}},
                                                          std::move(selector_no_16bit_and_positive_scale),
                                                          std::move(drop_action_no_int16_and_positive_scale));
 
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DropQDQNodesSelector>(true);
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DropQDQNodesSelector>(true, false, true, providers);
+  // DepthToSpace and SpaceToDepth not included because there are no integer implementations.
+  // https://github.com/microsoft/onnxruntime/issues/21287
   qdq_selector_action_registry.RegisterSelectorAndAction(drop_action_name,
-                                                         {{"Gather", {}},
+                                                         {{"Expand", {}},
+                                                          {"Flatten", {}},
+                                                          {"Gather", {}},
+                                                          {"GatherElements", {}},
                                                           {"Reshape", {}},
-                                                          {"Transpose", {}},
+                                                          {"Slice", {}},
                                                           {"Squeeze", {}},
+                                                          {"Tile", {}},
+                                                          {"Transpose", {}},
                                                           {"Unsqueeze", {}}},
                                                          std::move(selector),
                                                          std::move(drop_action));
@@ -108,7 +125,8 @@ void DropDQNodesRules(SelectorActionRegistry& qdq_selector_action_registry) {
 
 #if !defined(ORT_MINIMAL_BUILD)
   // TODO: Enable 16-bit types in selector when ArgMax supports 16-bit integer input tensors.
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DropDQNodesSelector>();
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DropDQNodesSelector>(false, false, providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"ArgMax", {}}},
                                                          std::move(selector),
@@ -191,7 +209,8 @@ void VariadicOpQDQRules(SelectorActionRegistry& qdq_selector_action_registry) {
 
 #if !defined(ORT_MINIMAL_BUILD)
   // TODO: Enable 16-bit types in selector when QLinearConcat supports 16-bit.
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::InputVariadicSelector>();
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::InputVariadicSelector>(false, false, providers);
 
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"Concat", {}}},
@@ -213,7 +232,11 @@ void ConvQDQRules(SelectorActionRegistry& qdq_selector_action_registry, bool is_
 
 #if !defined(ORT_MINIMAL_BUILD)
   // TODO: Enable 16-bit types in selector when QLinearConv supports 16-bit.
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::ConvSelector>(is_int8_allowed);
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::ConvSelector>(is_int8_allowed,
+                                                                               false,
+                                                                               false,
+                                                                               providers);
 
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"Conv", {}}},
@@ -236,7 +259,11 @@ void MatMulQDQRules(SelectorActionRegistry& qdq_selector_action_registry, bool i
 
 #if !defined(ORT_MINIMAL_BUILD)
   // TODO: Enable 16-bit types in selector when QLinearMatMul and MatMulInteger support 16-bit.
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::MatMulSelector>(is_int8_allowed);
+  std::vector<const char*> providers = {kCpuExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::MatMulSelector>(is_int8_allowed,
+                                                                                 false,
+                                                                                 false,
+                                                                                 providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"MatMul", {}}},
                                                          std::move(selector),
@@ -263,7 +290,8 @@ void DQMatMulToMatMulNBitsRules(SelectorActionRegistry& qdq_selector_action_regi
                                                          p_buffered_tensors);
 
 #if !defined(ORT_MINIMAL_BUILD)
-  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DQMatMulToMatMulNBitsSelector>();
+  std::vector<const char*> providers = {kCpuExecutionProvider, kCudaExecutionProvider, kDmlExecutionProvider};
+  std::unique_ptr<NodeSelector> selector = std::make_unique<QDQ::DQMatMulToMatMulNBitsSelector>(providers);
   qdq_selector_action_registry.RegisterSelectorAndAction(action_name,
                                                          {{"MatMul", {}}},
                                                          std::move(selector),
@@ -354,8 +382,9 @@ QDQSelectorActionTransformer::QDQSelectorActionTransformer(
           CreateSelectorActionRegistry(is_int8_allowed, qdq_matmulnbits_accuracy_level,
                                        intra_op_thread_pool, p_buffered_tensors),
           apply_context,
-          // this transformer is only compatible with the CPU and DML EP
-          {kCpuExecutionProvider, kDmlExecutionProvider}} {
+          // this transformer is compatible with CPU, DML, ACL and CUDA EP.
+          // There is further EP control on the rule level.
+          {kCpuExecutionProvider, kDmlExecutionProvider, kAclExecutionProvider, kCudaExecutionProvider}} {
 }
 
 }  // namespace onnxruntime
