@@ -5,13 +5,11 @@
 
 #include "core/framework/op_kernel.h"
 
-const void* JsepOutput(void* context, int index, void* data) {
-  uint32_t* data_offset = reinterpret_cast<uint32_t*>(data);
+const void* JsepOutput(void* context, int index, const void* data) {
+  const uint32_t* data_offset = reinterpret_cast<const uint32_t*>(data);
   uint32_t dim = *data_offset++;
   size_t dim_size = static_cast<size_t>(dim);
-  std::vector<int64_t> dims;
-  dims.reserve(dim_size);
-  dims.resize(dim_size);
+  std::vector<int64_t> dims(dim_size);
   for (size_t i = 0; i < dim_size; i++) {
     dims[i] = static_cast<int64_t>(*data_offset++);
   }
@@ -24,3 +22,8 @@ const void* JsepOutput(void* context, int index, void* data) {
   LOGF_DEFAULT(VERBOSE, "JsepOutput -- data=%zu", (size_t)(r));
   return r;
 }
+
+const void* JsepGetNodeName(const void* kernel) {
+  const auto& name = reinterpret_cast<const onnxruntime::OpKernel*>(kernel)->Node().Name();
+  return name.c_str();
+}
