@@ -25,10 +25,8 @@ struct QnnTensorInfo {
 
 class QnnModel {
  public:
-  QnnModel(const logging::Logger& logger,
-           QnnBackendManager* qnn_backend_manager)
-      : logger_(logger),
-        qnn_backend_manager_(qnn_backend_manager) {
+  QnnModel(QnnBackendManager* qnn_backend_manager)
+      : qnn_backend_manager_(qnn_backend_manager) {
     qnn_backend_type_ = qnn_backend_manager_->GetQnnBackendType();
   }
 
@@ -37,13 +35,15 @@ class QnnModel {
 
   Status ComposeGraph(const GraphViewer& graph_viewer,
                       const onnxruntime::Node& fused_node,
+                      const qnn::ModelSettings& model_settings,
+                      const logging::Logger& logger,
                       const QnnGraph_Config_t** graph_configs = nullptr);
 
-  Status FinalizeGraphs();
+  Status FinalizeGraphs(const logging::Logger& logger);
 
-  Status SetupQnnInputOutput();
+  Status SetupQnnInputOutput(const logging::Logger& logger);
 
-  Status ExecuteGraph(const Ort::KernelContext& context);
+  Status ExecuteGraph(const Ort::KernelContext& context, const logging::Logger& logger);
 
   const OnnxTensorInfo* GetOutputInfo(const std::string& name) const {
     auto it = outputs_info_.find(name);
@@ -55,11 +55,13 @@ class QnnModel {
   }
 
   Status SetGraphInputOutputInfo(const GraphViewer& graph_viewer,
-                                 const onnxruntime::Node& fused_node);
+                                 const onnxruntime::Node& fused_node,
+                                 const logging::Logger& logger);
   Status ParseGraphInputOrOutput(ConstPointerContainer<std::vector<NodeArg*>>& input_output_defs,
                                  std::vector<std::string>& input_output_names,
                                  std::unordered_map<std::string, OnnxTensorInfo>& input_output_info_table,
                                  std::unordered_map<std::string, size_t>& input_output_index,
+                                 const logging::Logger& logger,
                                  bool is_input = false);
 
   const std::unordered_set<std::string>& GetInitializerInputs() const { return initializer_inputs_; }
@@ -102,12 +104,12 @@ class QnnModel {
     return outputs_info_;
   }
 
-  const std::string& Name() { return graph_info_->Name(); }
+  const std::string& Name() const { return graph_info_->Name(); }
 
  private:
   const NodeUnit& GetNodeUnit(const Node* node,
                               const std::unordered_map<const Node*, const NodeUnit*>& node_unit_map) const;
-  bool GetGraphInfoFromModel(QnnModelWrapper& model_wrapper);
+  bool GetGraphInfoFromModel(QnnModelWrapper& model_wrapper, const logging::Logger& logger);
 
   Status GetQnnTensorDataLength(const std::vector<uint32_t>& dims,
                                 Qnn_DataType_t data_type,
@@ -125,7 +127,6 @@ class QnnModel {
   }
 
  private:
-  const logging::Logger& logger_;
   std::unique_ptr<GraphInfo> graph_info_;
   QnnBackendManager* qnn_backend_manager_ = nullptr;
   // <input_name, input_index>, initializer inputs are excluded, keep the input index here
