@@ -502,7 +502,7 @@ def run_cross_diff_seqlen_headsize_8():
     )
 
 
-def run_self_past_present_headsize_8_nomask_norelposbias():
+def run_self_past_present_headsize_8_nomask_no_attn_bias():
     hidden_dim = 16
     q_head_size = 8
     v_head_size = 8
@@ -554,8 +554,8 @@ def create_test_data():
     print("SelfAttention_Batch2_HeadSize32_PackedQKV")
     run_self_batch2_headsize_32_packed_qkv()
 
-    print("SelfAttention_WithPastAndPresent_HeadSize8_NoMask_NoRelPosBias")
-    run_self_past_present_headsize_8_nomask_norelposbias()
+    print("SelfAttention_WithPastAndPresent_HeadSize8_NoMask_NoAttnBias")
+    run_self_past_present_headsize_8_nomask_no_attn_bias()
 
     print("CrossAttention_DiffSequenceLengths_HeadSize8")
     run_cross_diff_seqlen_headsize_8()
