@@ -5,11 +5,9 @@
 
 #include <benchmark/benchmark.h>
 
+#include <functional>
 #include <random>
 
-void ArgsProduct(benchmark::internal::Benchmark* bench,
-                 const std::vector<std::vector<int64_t>>& arglists);
-
 template <typename ElementType>
 std::vector<ElementType> RandomVectorUniform(
     size_t N,
