@@ -1,10 +1,8 @@
 #!/bin/bash
-# Copyright (c) Microsoft Corporation. All rights reserved.
-# Licensed under the MIT License.
 
 pip3 install --user --upgrade pip
 
-pip3 install --user numpy==1.19.0 torch pytest
+pip3 install --user numpy torch pytest
 pip3 install --user /build/Release/dist/*.whl
 
 export PYTHONPATH=/onnxruntime_src/tools:/usr/local/lib/python3.8/site-packages:$PYTHONPATH
